// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Management.Scheduler;
using Microsoft.WindowsAzure.Management.Scheduler.Models;

namespace Microsoft.WindowsAzure.Management.Scheduler
{
    public static partial class JobCollectionOperationsExtensions
    {
        /// <summary>
        /// Create a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service containing the job collection.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to create.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Create Job Collection operation.
        /// </param>
        /// <returns>
        /// The Create Job Collection operation response.
        /// </returns>
        public static JobCollectionCreateResponse BeginCreating(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName, JobCollectionCreateParameters parameters)
        {
            try
            {
                return operations.BeginCreatingAsync(cloudServiceName, jobCollectionName, parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Create a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service containing the job collection.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to create.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Create Job Collection operation.
        /// </param>
        /// <returns>
        /// The Create Job Collection operation response.
        /// </returns>
        public static Task<JobCollectionCreateResponse> BeginCreatingAsync(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName, JobCollectionCreateParameters parameters)
        {
            return operations.BeginCreatingAsync(cloudServiceName, jobCollectionName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static OperationResponse BeginDeleting(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName)
        {
            try
            {
                return operations.BeginDeletingAsync(cloudServiceName, jobCollectionName).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Deletes a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<OperationResponse> BeginDeletingAsync(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName)
        {
            return operations.BeginDeletingAsync(cloudServiceName, jobCollectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// Update a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service containing the job collection.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Job Collection operation.
        /// </param>
        /// <returns>
        /// The Update Job Collection operation response.
        /// </returns>
        public static JobCollectionUpdateResponse BeginUpdating(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName, JobCollectionUpdateParameters parameters)
        {
            try
            {
                return operations.BeginUpdatingAsync(cloudServiceName, jobCollectionName, parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Update a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service containing the job collection.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Job Collection operation.
        /// </param>
        /// <returns>
        /// The Update Job Collection operation response.
        /// </returns>
        public static Task<JobCollectionUpdateResponse> BeginUpdatingAsync(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName, JobCollectionUpdateParameters parameters)
        {
            return operations.BeginUpdatingAsync(cloudServiceName, jobCollectionName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Determine if the JobCollection name is available to be used.
        /// JobCollection names must be unique within a cloud-service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service.
        /// </param>
        /// <param name='jobCollectionName'>
        /// A name for the JobCollection. The name must be unique as scoped
        /// within the CloudService.  The name can be up to 100 characters in
        /// length.
        /// </param>
        /// <returns>
        /// The Check Name Availability operation response.
        /// </returns>
        public static JobCollectionCheckNameAvailabilityResponse CheckNameAvailability(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName)
        {
            try
            {
                return operations.CheckNameAvailabilityAsync(cloudServiceName, jobCollectionName).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Determine if the JobCollection name is available to be used.
        /// JobCollection names must be unique within a cloud-service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service.
        /// </param>
        /// <param name='jobCollectionName'>
        /// A name for the JobCollection. The name must be unique as scoped
        /// within the CloudService.  The name can be up to 100 characters in
        /// length.
        /// </param>
        /// <returns>
        /// The Check Name Availability operation response.
        /// </returns>
        public static Task<JobCollectionCheckNameAvailabilityResponse> CheckNameAvailabilityAsync(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName)
        {
            return operations.CheckNameAvailabilityAsync(cloudServiceName, jobCollectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// Create a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service containing the job collection.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to create.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Create Job Collection operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static SchedulerOperationStatusResponse Create(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName, JobCollectionCreateParameters parameters)
        {
            try
            {
                return operations.CreateAsync(cloudServiceName, jobCollectionName, parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Create a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service containing the job collection.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to create.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Create Job Collection operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<SchedulerOperationStatusResponse> CreateAsync(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName, JobCollectionCreateParameters parameters)
        {
            return operations.CreateAsync(cloudServiceName, jobCollectionName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to delete.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static SchedulerOperationStatusResponse Delete(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName)
        {
            try
            {
                return operations.DeleteAsync(cloudServiceName, jobCollectionName).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Deletes a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to delete.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<SchedulerOperationStatusResponse> DeleteAsync(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName)
        {
            return operations.DeleteAsync(cloudServiceName, jobCollectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// Retreive a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// Name of the cloud service.
        /// </param>
        /// <param name='jobCollectionName'>
        /// Name of the job collection.
        /// </param>
        /// <returns>
        /// The Get Job Collection operation response.
        /// </returns>
        public static JobCollectionGetResponse Get(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName)
        {
            try
            {
                return operations.GetAsync(cloudServiceName, jobCollectionName).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Retreive a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// Name of the cloud service.
        /// </param>
        /// <param name='jobCollectionName'>
        /// Name of the job collection.
        /// </param>
        /// <returns>
        /// The Get Job Collection operation response.
        /// </returns>
        public static Task<JobCollectionGetResponse> GetAsync(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName)
        {
            return operations.GetAsync(cloudServiceName, jobCollectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// Update a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service containing the job collection.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Job Collection operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static SchedulerOperationStatusResponse Update(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName, JobCollectionUpdateParameters parameters)
        {
            try
            {
                return operations.UpdateAsync(cloudServiceName, jobCollectionName, parameters).Result;
            }
            catch (AggregateException ex)
            {
                if (ex.InnerExceptions.Count > 1)
                {
                    throw;
                }
                else
                {
                    throw ex.InnerException;
                }
            }
        }
        
        /// <summary>
        /// Update a job collection.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Scheduler.IJobCollectionOperations.
        /// </param>
        /// <param name='cloudServiceName'>
        /// The name of the cloud service containing the job collection.
        /// </param>
        /// <param name='jobCollectionName'>
        /// The name of the job collection to update.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Update Job Collection operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself.  If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request.  If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request, and also includes error
        /// information regarding the failure.
        /// </returns>
        public static Task<SchedulerOperationStatusResponse> UpdateAsync(this IJobCollectionOperations operations, string cloudServiceName, string jobCollectionName, JobCollectionUpdateParameters parameters)
        {
            return operations.UpdateAsync(cloudServiceName, jobCollectionName, parameters, CancellationToken.None);
        }
    }
}
