// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Management.Store.Models;

namespace Microsoft.WindowsAzure.Management.Store.Models
{
    /// <summary>
    /// The response structure for the Cloud Service List operation.
    /// </summary>
    public partial class CloudServiceListResponse : OperationResponse, IEnumerable<CloudServiceListResponse.CloudService>
    {
        private IList<CloudServiceListResponse.CloudService> _cloudServices;
        
        /// <summary>
        /// The list of cloud service locations for this subscription.
        /// </summary>
        public IList<CloudServiceListResponse.CloudService> CloudServices
        {
            get { return this._cloudServices; }
            set { this._cloudServices = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the CloudServiceListResponse class.
        /// </summary>
        public CloudServiceListResponse()
        {
            this._cloudServices = new List<CloudServiceListResponse.CloudService>();
        }
        
        /// <summary>
        /// Gets the sequence of CloudServices.
        /// </summary>
        public IEnumerator<CloudServiceListResponse.CloudService> GetEnumerator()
        {
            return this.CloudServices.GetEnumerator();
        }
        
        /// <summary>
        /// Gets the sequence of CloudServices.
        /// </summary>
        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
        
        /// <summary>
        /// Defines a cloud service-supporting region in which a storm item is
        /// located.
        /// </summary>
        public partial class CloudService
        {
            private string _description;
            
            /// <summary>
            /// The description of the cloud service region.
            /// </summary>
            public string Description
            {
                get { return this._description; }
                set { this._description = value; }
            }
            
            private string _geoRegion;
            
            /// <summary>
            /// The geographical region in which this cloud service can run.
            /// </summary>
            public string GeoRegion
            {
                get { return this._geoRegion; }
                set { this._geoRegion = value; }
            }
            
            private string _label;
            
            /// <summary>
            /// The label of the cloud service region.
            /// </summary>
            public string Label
            {
                get { return this._label; }
                set { this._label = value; }
            }
            
            private string _name;
            
            /// <summary>
            /// The name of the cloud service region.
            /// </summary>
            public string Name
            {
                get { return this._name; }
                set { this._name = value; }
            }
            
            private IList<CloudServiceListResponse.CloudService.AddOnResource> _resources;
            
            /// <summary>
            /// A list of existing store resources installed into a cloud
            /// service region.
            /// </summary>
            public IList<CloudServiceListResponse.CloudService.AddOnResource> Resources
            {
                get { return this._resources; }
                set { this._resources = value; }
            }
            
            /// <summary>
            /// Initializes a new instance of the CloudService class.
            /// </summary>
            public CloudService()
            {
                this._resources = new List<CloudServiceListResponse.CloudService.AddOnResource>();
            }
            
            /// <summary>
            /// A store add-on item.
            /// </summary>
            public partial class AddOnResource
            {
                private string _eTag;
                
                /// <summary>
                /// The ETag for this store resource.
                /// </summary>
                public string ETag
                {
                    get { return this._eTag; }
                    set { this._eTag = value; }
                }
                
                private string _name;
                
                /// <summary>
                /// The user-input name of this store item.
                /// </summary>
                public string Name
                {
                    get { return this._name; }
                    set { this._name = value; }
                }
                
                private string _namespace;
                
                /// <summary>
                /// The namespace in which this store item resides.
                /// </summary>
                public string Namespace
                {
                    get { return this._namespace; }
                    set { this._namespace = value; }
                }
                
                private IDictionary<string, string> _outputItems;
                
                /// <summary>
                /// Output items associated with an individual store resource.
                /// </summary>
                public IDictionary<string, string> OutputItems
                {
                    get { return this._outputItems; }
                    set { this._outputItems = value; }
                }
                
                private string _plan;
                
                /// <summary>
                /// The plan for this store item as selected by the user.
                /// </summary>
                public string Plan
                {
                    get { return this._plan; }
                    set { this._plan = value; }
                }
                
                private string _schemaVersion;
                
                /// <summary>
                /// The schema version for this resource.
                /// </summary>
                public string SchemaVersion
                {
                    get { return this._schemaVersion; }
                    set { this._schemaVersion = value; }
                }
                
                private string _state;
                
                /// <summary>
                /// The state of this store resource.
                /// </summary>
                public string State
                {
                    get { return this._state; }
                    set { this._state = value; }
                }
                
                private CloudServiceListResponse.CloudService.AddOnResource.OperationStatus _status;
                
                /// <summary>
                /// Operation status items associated with an individual store
                /// resource.
                /// </summary>
                public CloudServiceListResponse.CloudService.AddOnResource.OperationStatus Status
                {
                    get { return this._status; }
                    set { this._status = value; }
                }
                
                private string _type;
                
                /// <summary>
                /// The type of store item.
                /// </summary>
                public string Type
                {
                    get { return this._type; }
                    set { this._type = value; }
                }
                
                private IList<CloudServiceListResponse.CloudService.AddOnResource.UsageLimit> _usageLimits;
                
                /// <summary>
                /// Usage meters associated with an individual store resource.
                /// </summary>
                public IList<CloudServiceListResponse.CloudService.AddOnResource.UsageLimit> UsageLimits
                {
                    get { return this._usageLimits; }
                    set { this._usageLimits = value; }
                }
                
                /// <summary>
                /// Initializes a new instance of the AddOnResource class.
                /// </summary>
                public AddOnResource()
                {
                    this._outputItems = new Dictionary<string, string>();
                    this._usageLimits = new List<CloudServiceListResponse.CloudService.AddOnResource.UsageLimit>();
                }
                
                /// <summary>
                /// The operation status of an individual store resource item.
                /// </summary>
                public partial class OperationStatus
                {
                    private string _result;
                    
                    /// <summary>
                    /// The result of this operation status.
                    /// </summary>
                    public string Result
                    {
                        get { return this._result; }
                        set { this._result = value; }
                    }
                    
                    private string _type;
                    
                    /// <summary>
                    /// The type of this operation status.
                    /// </summary>
                    public string Type
                    {
                        get { return this._type; }
                        set { this._type = value; }
                    }
                    
                    /// <summary>
                    /// Initializes a new instance of the OperationStatus class.
                    /// </summary>
                    public OperationStatus()
                    {
                    }
                }
                
                /// <summary>
                /// Describes the current utilization and metering of a store
                /// resource item.
                /// </summary>
                public partial class UsageLimit
                {
                    private long _amountIncluded;
                    
                    /// <summary>
                    /// Defines the limit of this usage included in this store
                    /// resource's plan.
                    /// </summary>
                    public long AmountIncluded
                    {
                        get { return this._amountIncluded; }
                        set { this._amountIncluded = value; }
                    }
                    
                    private long _amountUsed;
                    
                    /// <summary>
                    /// The amount of this store resource that has already been
                    /// used.
                    /// </summary>
                    public long AmountUsed
                    {
                        get { return this._amountUsed; }
                        set { this._amountUsed = value; }
                    }
                    
                    private string _name;
                    
                    /// <summary>
                    /// The name of this usage limit.
                    /// </summary>
                    public string Name
                    {
                        get { return this._name; }
                        set { this._name = value; }
                    }
                    
                    private string _unit;
                    
                    /// <summary>
                    /// The unit in which this usage limit is measured.
                    /// </summary>
                    public string Unit
                    {
                        get { return this._unit; }
                        set { this._unit = value; }
                    }
                    
                    /// <summary>
                    /// Initializes a new instance of the UsageLimit class.
                    /// </summary>
                    public UsageLimit()
                    {
                    }
                }
            }
        }
    }
}
